/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "oirSupport.h"

void minrtsLudCmp(float *a, int n, int *indx, float *d) {

   int i, imax,j,k;
   float big,dum,sum,temp;
   float *vv;

   vv = (float *)malloc(n*sizeof(float));

   *d=1.;

   for (i=0;i<n;i++) {
      big = 0.0;
      for (j=0;j<n;j++) {
         temp = fabs(a[n*i+j]);
         if (temp > big) big = temp;
      }
      vv[i] = 1./big;
   }

   for (j=0; j<n;j++){
      for (i=0;i<j;i++) {
         sum = a[n*i+j];
         for (k=0;k<i;k++) sum -= a[i*n+k]*a[k*n+j];
         a[i*n+j] = sum;
      }
      big = 0.0;
      for (i=j;i<n;i++) {
         sum = a[n*i+j];
         for (k=0;k<j;k++) sum -= a[n*i+k] * a[n*k+j];
         a[n*i+j] = sum;
         dum = vv[i]*fabs(sum);
         if (dum >=big) {
            big = dum;
            imax = i;
         }
      }
      if (j != imax) {
         for (k=0;k<n;k++) {
            dum=a[n*imax+k];
            a[n*imax+k] = a[n*j+k];
            a[n*j+k] = dum;
         }
         *d = -(*d);
         vv[imax] = vv[j];
      }indx[j] = imax;
      if (a[n*j+j] == 0.0) a[n*j+j] = TINY;

      if (j != (n-1)) {
         dum = 1./(a[j*n+j]);
         for (i=j+1;i<n;i++) a[n*i+j] *= dum;
      }
   }
   free(vv);
   return;
}

void minrtsLubKsb(float *a, int n, int *indx, float *b) {
   int i,ii,ip,j;
   float sum;

   ii = -1;
   for (i=0;i<n;i++) {
      ip = indx[i];
      sum = b[ip];
      b[ip] = b[i];
      if (ii >= 0)
         for (j=ii;j<=i-1;j++) sum -= a[n*i+j]*b[j];
      else if (sum) ii=i;
      b[i] = sum;
   }
   for (i=n-1;i>=0;i--) {
      sum = b[i];
      for (j=i+1;j<n;j++) sum -=a[n*i+j]*b[j];
      b[i] = sum/a[n*i+i];
   }
   return;
}


void minrtsPolyFit(int order, int nData, float *xCoord, float *data, 
   float *weight, float *coeff) {

   float *matrix, *vector;
   float d, pj, pk; 
   int *indx;
   int i,j,k;

   matrix = (float *)malloc((1+order)*(1+order)*sizeof(float));
   vector = (float *)malloc((1+order)*sizeof(float));
   indx = (int *)malloc((1+order)*sizeof(float));
   memset(matrix, 0, (1+order)*(1+order)*sizeof(float));
   memset(vector, 0, (1+order)*sizeof(float));

   for (i=0; i<nData; i++) {
      for (j=0;j<=order; j++) {
         if (j == 0) pj = 1; else pj = pow(xCoord[i], j);
         vector[j] += data[i]*weight[i]*pj;
         for (k=0;k<=order; k++) {
            if (k == 0) pk = 1; else pk = pow(xCoord[i], k);
            matrix[k+(1+order)*j] += weight[i] * pj * pk;
         }
      }
   }
   minrtsLudCmp(matrix, order+1, indx, &d);
   minrtsLubKsb(matrix, order+1, indx, vector);
   for (i=0;i<=order;i++) coeff[i] = vector[i];

   free(matrix);
   free(vector);
   free(indx);
   return;
   }

